/*
 * This file is part of Adblock Plus <https://adblockplus.org/>,
 * Copyright (C) 2006-present eyeo GmbH
 *
 * Adblock Plus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * Adblock Plus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
 */

{
  var maxSurveyNodes = 65536;
  var maxSurveyTimeSlice = 4;
  var maxSurveyBuffer = 64;
  var reWhitespace = /\s/;

  var idFromNode = (node, out) => {
    var raw = node.id;
    if ( typeof raw !== 'string' || raw.length === 0 ) { return; }
    var s = raw.trim();
    if ( out.includes(s) || s.length === 0 ) { return; }
    out.push(s);
  }

  var classesFromNode = (node, out) => {
    var s = node.className;
    if ( typeof s !== 'string' || s.length === 0 ) { return; }
    if ( reWhitespace.test(s) === false ) {
      if (out.includes(s) ) { return; }
      out.push(s);
      return;
    }
    for ( var s of node.classList.values() ) {
      if ( out.includes(s) ) { continue; }
      out.push(s);
    }
  }

  var queryClassAndId = function () {
    var queriedIds = [];
    var queriedClasses = [];

    var pendingNodes = {
      nodeLists: [],
      buffer: [
          null, null, null, null, null, null, null, null,
          null, null, null, null, null, null, null, null,
          null, null, null, null, null, null, null, null,
          null, null, null, null, null, null, null, null,
          null, null, null, null, null, null, null, null,
          null, null, null, null, null, null, null, null,
          null, null, null, null, null, null, null, null,
          null, null, null, null, null, null, null, null,
      ],
      j: 0,
      accepted: 0,
      iterated: 0,
      stopped: false,
      add(nodes) {
        if ( nodes.length === 0 || this.accepted >= maxSurveyNodes ) {
          return;
        }
        this.nodeLists.push(nodes);
        this.accepted += nodes.length;
      },
      next() {
        if ( this.nodeLists.length === 0 || this.stopped ) { return 0; }
        var nodeLists = this.nodeLists;
        var ib = 0;
        do {
          var nodeList = nodeLists[0];
          var j = this.j;
          var n = j + maxSurveyBuffer - ib;
          if ( n > nodeList.length ) {
            n = nodeList.length;
          }
          for ( var i = j; i < n; i++ ) {
            this.buffer[ib++] = nodeList[j++];
          }
          if ( j !== nodeList.length ) {
            this.j = j;
            break;
          }
          this.j = 0;
          this.nodeLists.shift();
        } while ( ib < maxSurveyBuffer && nodeLists.length !== 0 );
        this.iterated += ib;
        if ( this.iterated >= maxSurveyNodes ) {
          this.nodeLists = [];
          this.stopped = true;
        }
        return ib;
      },
      hasNodes() {
        return this.nodeLists.length !== 0;
      }
    }

    try {
      pendingNodes.add(document.querySelectorAll('[id]:not(html):not(body),[class]:not(html):not(body)'));

      var t0 = performance.now();
      var nodes = pendingNodes.buffer;
      var deadline = t0 + maxSurveyTimeSlice;
      for (;;) {
        var n = pendingNodes.next();
        if ( n === 0 ) { break; }
        for ( var i = 0; i < n; i++ ) {
          var node = nodes[i]; nodes[i] = null;
          idFromNode(node, queriedIds);
          classesFromNode(node, queriedClasses);
        }
        if ( performance.now() >= deadline ) { break; }
      }
    } catch (e) {}

    return {
      classList: queriedClasses,
      idList: queriedIds
    }
  }

//  var classAndId = queryClassAndId();

  {{DEBUG}} console.log('starting injecting eh css rules for ' + document.location.href);

//  if (classAndId.classList.length > 0 || classAndId.idList.length > 0) {
   {{BRIDGE}}.getElemhideStyleSheet(document.location.href, JSON.stringify([]), JSON.stringify([]));
//  }
}